<?php
// $Id: biblio.inc,v 1.1.2.1 2006/11/26 19:38:03 agentken Exp $
// Biblio include by csc4.  http://drupal.org/node/98029

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

// static variables for this type
define(BIBLIO_THRESHOLD, 10);
define(BIBLIO_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_biblio($get_options = TRUE) {
  if(module_exist('biblio')) {
    $type = array(
      'name' => t('biblios'),
      'description' => t('<b>Bibliography</b>: All bibliographies from a specific user'),
      'include' => 'biblio',    
      'prefix' => t(''),
      'suffix' => t('bibliographies'),
      'weight' => -10,
      'form' => FALSE,
      'label' => t('Add Bibliography'),
      'help' => t('You can choose bibliography postings by specific users. Type a user name in the search box, or choose from a recently active bibliographer.'),
      'search' => TRUE,
      'threshold' => BIBLIO_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_biblio_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type
 */
function mysite_type_biblio_options() {
  $options = array();
  // can't respect biblio permissions because authenticated users aren't in the users_roles table
  // but it only pulls users who have bibliographies.
  $sql = "SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'biblio' ORDER BY n.nid DESC";
  $result = db_query_range($sql, $implode, BIBLIO_FROM, BIBLIO_THRESHOLD);
  $biblios = array();
  while($item = db_fetch_object($result)) {
    $biblios[] =$item;
  }
  foreach ($biblios as $key => $value) {
    $options['name'][] = $value->name . t("'s");
    $options['type_id'][] = $value->uid;
    $options['type'][] = 'biblio';
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_biblio_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if (is_null($title)) {
      $biblio = user_load(array('uid' => $type_id));
      $title = $biblio->name;
    }  
    $type = mysite_type_biblio();
    $title = $type['prefix'] . ' ' . $title . t("'s") . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_biblio_data($type_id = NULL) {
  if(!empty($type_id)) {
    
    $sql = "SELECT nid FROM {node} WHERE type = 'biblio' AND status = 1 AND uid = %d ORDER BY sticky DESC, created DESC";
    $result = db_query_range($sql, $type_id, 0, variable_get('mysite_elements', 5));

    $data = array(
      'base' => 'biblio/' . $type_id,
      'xml' => 'biblio/' . $type_id . '/feed',
      );
    $items = array();
    $i = 0;
    while ($nid = db_fetch_object($result)) {
      $node = node_load(array('nid' => $nid->nid));
      $items[$i]['type'] = $node->type;
      $items[$i]['link'] = l($node->title, 'node/' . $nid->nid);
      $items[$i]['title'] = check_plain($node->title);
      $items[$i]['subtitle'] = NULL;      
      $items[$i]['date'] = $node->changed;
      $items[$i]['uid'] = $node->uid;
      $items[$i]['author'] = check_plain($node->name);
      $items[$i]['teaser'] = check_markup($node->teaser, $node->format);    
      $items[$i]['nid'] = $node->nid;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  

}

/**
 * Return block settings for this type
 */
function mysite_type_biblio_block($arg, $op = 'view') {
  global $user;
  if (user_access('edit mysite') && ($arg[0] == 'biblio' || $arg[0] == 'user') && is_numeric($arg[1])) {
    $biblio = user_load(array('uid' => $arg[1]));
    if (user_access('edit own bibliography', $biblio)) {
      $data = array();
      $data['uid'] = $user->uid;
      $data['type'] = 'biblio';
      $data['type_id'] = $biblio->uid;
      $data['title'] = mysite_type_biblio_title($biblio->uid, $biblio->name);
      $content = mysite_block_handler($data);
      return $content;
    }  
  }
}

/**
 * Search options for this type
 */
function mysite_type_biblio_search($uid = NULL) {
  if (!is_null($uid)) {
    $form['add_biblio']['biblio_title'] = array('#type' => 'textfield',
      '#title' => t('User Name'),
      '#default_value' => $edit['biblio_title'],
      '#maxlength' => 64,
      '#description' => t('The user name of the author of the bibliography you wish to add.'),
      '#required' => FALSE, // this must be false, since all the search for are the same form
      '#autocomplete_path' => 'autocomplete/mysite/biblio'
    );  
    $form['add_biblio']['uid'] = array('#type' => 'hidden', '#value' => $uid);
    $form['add_biblio']['type'] = array('#type' => 'hidden', '#value' => 'biblio');
    $form['add_biblio']['submit'] = array('#type' => 'submit', '#value' =>t('Add biblio'));  
    $output .= drupal_get_form('mysite_type_biblio_search', $form);  
    return $output;
  }
}

/**
 * Form handler for search
 */
function mysite_type_biblio_search_submit($form_id, $form_values) {
  // we use LIKE here in case JavaScript autocomplete support doesn't work.
  // or in case the user doesn't autocomplete the form
  $result = db_query("SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'biblio' AND u.name LIKE LOWER('%s%%')", $form_values['biblio_title']);
  $count = 0;
  while ($biblio = db_fetch_object($result)) {
    $data[$count]['uid'] = $form_values['uid'];
    $data[$count]['type'] = $form_values['type'];
    $data[$count]['type_id'] = $biblio->uid;
    $data[$count]['title'] = mysite_type_biblio_title($biblio->uid, $biblio->name);
    $data[$count]['description'] = t('The bibliographies of %user', array('%user' => $biblio->name));    
    $count++;
  }
  // pass the $data to the universal handler
  mysite_search_handler($data, 'biblio');
  return;
}

/**
 * AJAX autocomplete for biblio titles
 * The include files have no menu callbacks!
 */
function mysite_type_biblio_autocomplete($string) {
  $matches = array();
  $result = db_query_range("SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'biblio' AND u.name LIKE LOWER('%s%%')", $string, 0, 10);
  while ($biblio = db_fetch_object($result)) {
    $matches[$biblio->name] = check_plain($biblio->name);
  }
  return $matches;
}
 
/**
 * Cron cleanup function
 */
function mysite_type_biblio_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT DISTINCT(u.uid) FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'biblio' AND u.uid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id'], $implode));
    if(empty($check->uid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}
 
