<?php
// $Id: refine.inc,v 1.1.2.4 2006/11/26 19:25:19 agentken Exp $
// Addresses http://drupal.org/node/93060

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

// static variables for this type
define(REFINE_THRESHOLD, 10);
define(REFINE_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_refine($get_options = TRUE) {
  if(module_exist('refine_by_taxo') && module_exist('taxonomy')) {
    $type = array(
      'name' => t('content groups'),
      'description' => t('<b>Refined Categories</b>: Posts that match multiple terms.'),
      'include' => 'refine',    
      'prefix' => t(''),
      'suffix' => t('articles'),
      'weight' => 10,
      'form' => FALSE,
      'label' => t('Add Content'),
      'help' => t('Display posts tagged with multiple categories.'),
      'search' => FALSE,
      'threshold' => REFINE_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_refine_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type
 */
function mysite_type_refine_options() {
  $options = array();
  // this will come later
  $data = mysite_get_custom('refine', NULL, TRUE);
  // since $all = TRUE, this returns an array
  if (is_array($data) && !empty($data)) {
    foreach ($data as $refine) {
      if ($i < REFINE_THRESHOLD) {
        $options['group'][] = '';
        $options['name'][] = $refine->title;
        $options['type_id'][] = $refine->myid;
        $options['type'][] = 'refine';
        $i++;
      }        
    }
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_refine_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if(is_null($title)) {
      $data = mysite_get_custom('refine', $type_id, FALSE);
      $title = $data->title;
    }  
    $type = mysite_type_refine();
    $title = $type['prefix'] . ' ' . $title . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_refine_data($type_id = NULL) {
  if(!empty($type_id)) {
    $tids = '';
    $data = mysite_get_custom('refine', $type_id);
    $terms = explode(',', $data->type_key);
    $result = taxonomy_select_nodes($terms, $operator = 'and', $depth = 0, $pager = FALSE, $order = 'n.changed DESC');
    $nids = array();
    while ($row = db_fetch_object($result)) {
      $nids[] = $row->nid;
    }    
    // slice the array to limit results
    $nodes = array_slice($nids, 0, variable_get('mysite_elements', 5));    
    $data = array(
      'base' => 'taxonomy/term/' . $data->type_key,
      'xml' => 'taxonomy/term/' . $data->type_key . '/0/feed'
      );
    $items = array();
    $i = 0;
    // we can't use node_view() here, because aggregator doesn't use nodes
    // and we need the same data structure for all content
    foreach ($nodes as $nid) {
      $node = node_load($nid);
      $items[$i]['type'] = $node->type;
      $items[$i]['link'] = l($node->title, 'node/' . $node->nid);
      $items[$i]['title'] = check_plain($node->title);
      $items[$i]['subtitle'] = NULL;         
      $items[$i]['date'] = $node->changed;
      $items[$i]['uid'] = $node->uid;
      $items[$i]['author'] = check_plain($node->name);
      $items[$i]['teaser'] = check_markup($node->teaser, $node->format);
      $items[$i]['nid'] = $node->nid;      
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  
}

/**
 * Return block settings for this type
 */
function mysite_type_refine_block($arg, $op = 'view') {
  global $user;
  // refine-by-taxo uses the symtax /taxonomy/term/1,2,4
  $terms = explode(',', $arg[2]);
  $name = '';
  if (user_access('access content') &&  $arg[0] == 'taxonomy' && $arg[1] == 'term' && count($terms) > 1) {
    foreach ($terms as $term) {
      $taxo[$term] = taxonomy_get_term($term);
      $name .= $taxo[$term]->name . ', ';
    }  
    $name = rtrim($name, ', ');
    $myid = mysite_get_myid('refine', $arg[2], $name);
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'refine';
    $data['type_id'] = $myid;
    $data['title'] = mysite_type_refine_title($myid, NULL);
    $content = mysite_block_handler($data);
    return $content;
  }
}

/**
 * Cron cleanup function
 */
function mysite_type_refine_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  // the join statement for items using strings for keys.
  $sql = "SELECT md.mid, md.uid, md.type_id, md.title, mc.type_key FROM {mysite_data} md INNER JOIN {mysite_content} mc ON mc.myid = md.type_id WHERE md.type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT tid FROM {term_data} WHERE tid IN (%s)";
    $check = db_fetch_object(db_query($sql, $item['type_key']));
    if(empty($check->tid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}

/**
 * Settings for this type.  
 * We can't use system settings, since we are not saving variables.
 */
function mysite_type_refine_settings() {
  $output = '';
  // get the vocabularies allowed by MySite
  $vocabularies = variable_get('mysite_term_vocabularies', NULL);
  $vform['refine'] = array(
    '#type' => 'fieldset',
    '#title' => t('Add Content Group'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('<p>Select as many terms as you like to create a new content view.  These items will show all content that matches term1 AND term2 AND so forth.</p>')
  );  
  $vform['refine']['title'] = array(
    '#type' => 'textfield',
    '#size' => 30,
    '#title' => t('Group Name')
  );
  if (!empty($vocabularies)) {
    foreach ($vocabularies as $vid) {
      $vocabulary[$vid] = taxonomy_get_vocabulary($vid);      
      $terms = taxonomy_get_tree($vid);
      if (!empty($terms)) {
        foreach ($terms as $term) {
          $options[$vid][$term->tid] = check_plain($term->name);
        }
      }
      $vform['refine'][$vid]['terms'] = array(
        '#type' => 'select', '#title' => check_plain($vocabulary[$vid]->name), '#default_value' => NULL,
        '#options' =>  $options[$vid],
        '#multiple' => TRUE,
        '#description' => t('Create an overview page with links to all custom user mysites?')
      );  
    }
  }
  $vform['refine']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Create New Group')
  );
  $output .= drupal_get_form('mysite_refine_add', $vform);
  $output .= t('<h3>Rename Groups</h3>');
  $output .= t('<p>The following content groups have been created by the Refine-By-Taxo module.</p>');
  $header = array(
    array('data' => t('ID #'), 'field' =>'myid'), 
    array('data' => t('Title'), 'field' => 'title'), 
    array('data' => t('Terms'), 'field' => 'type_key'),
    array('data' => t('Rename'))
  );
  $limit = 25;
  $sql = "SELECT myid, type_key, title FROM {mysite_content} WHERE type = 'refine'" . tablesort_sql($header);
  $result = pager_query($sql, $limit);   
  $rows = array();
  while ($group = db_fetch_object($result)) {
    $form[$group->myid]['mysite_refine']['title'] = array(
      '#type' => 'textfield',
      '#size' => '15',
      '#default_value' => NULL
    );
    $form[$group->myid]['mysite_refine']['myid'] = array(
      '#type' => 'hidden',
      '#value' => $group->myid
    );        
    $form[$group->myid]['mysite_refine']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Rename')
    );    
    $rows[] = array('data' => array($group->myid, l($group->title, 'taxonomy/term/' . $group->type_key), $group->type_key, drupal_get_form('mysite_refine_settings', $form[$group->myid])));
  }
  if (!empty($rows)) {
    $output .= theme_table($header, $rows);
    $output .= theme('pager', NULL, $limit, 0);      
  }
  return $output;
}

/**
 * Form validation for mysite_refine_settings
 */
function mysite_refine_settings_validate($form_id, $form_values) {
  if (!empty($form_values['title']) && is_string($form_values['title'])) {
    $sql = "SELECT myid FROM {mysite_content} WHERE lower(title) = '%s' AND type = 'refine'";
    $check = db_fetch_object(db_query($sql, trim(strtolower($form_values['title']))));
    if ($check->myid > 0) {
      form_set_error($form_values['myid'], t('The title must be unique.')); 
    }
    else {
      return;
    }  
  }
  else {
    form_set_error($form_values['myid'], t('You must enter at least one title.'));
  }
}

/**
 * Form submit for mysite_refine_settings
 */
function mysite_refine_settings_submit($form_id, $form_values) {
  $sql = "UPDATE {mysite_content} SET title = '%s' WHERE myid = %d AND type = 'refine'";
  db_query($sql, $form_values['title'], $form_values['myid']);
}

/**
 * Theme function for the form
 */
function theme_mysite_refine_settings($form) {
  $output = '<div id="mysite-form">';
  $output .= form_render($form);
  $output .= '</div>';
  return $output;  
}

/**
 * Form validation for mysite_refine_add
 */
function mysite_refine_add_validate($form_id, $form_values) {
  if (!empty($form_values['title']) && is_string($form_values['title'])) {  
    $sql = "SELECT myid FROM {mysite_content} WHERE lower(title) = '%s' AND type = 'refine'";
    $check = db_fetch_object(db_query($sql, trim(strtolower($form_values['title']))));
    if ($check->myid > 0) {
      form_set_error($form_values['title'], t('The title must be unique.')); 
    }
    else {
      if (count($form_values['terms']) < 2) {
        form_set_error($form_values['terms'], t('You must select at least two terms.'));       
      }
      else {
        ksort($form_values['terms']);
        $terms = implode(',', $form_values['terms']);
        $sql = "SELECT myid FROM {mysite_content} WHERE type_key = '%s' AND type = '%s'";
        $data = db_fetch_object(db_query($sql, $terms, 'refine'));      
        if ($data->myid > 0) {
          form_set_error($form_values['terms'], t('This content group already exists.'));       
        }
        else {
          return;
        }
      }  
    }  
  }
  else {
    form_set_error($form_values['title'], t('You must enter at a title.'));
  }  
}

/**
 * Form validation for mysite_refine_add
 */
function mysite_refine_add_submit($form_id, $form_values) {
  ksort($form_values['terms']);
  $terms = implode(',', $form_values['terms']);
  $myid = mysite_get_myid('refine', $terms, $form_values['title']);
  if ($myid > 0) {
    drupal_set_message(t('New content group added.'));
  }
  else {
    drupal_set_message(t('Insert failed'), 'error');
  }
}  