<?php
// $Id: aggregator.inc,v 1.1.2.2 2006/11/10 16:08:40 agentken Exp $

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

// static variables for this type
define(AGGREGATOR_THRESHOLD, 10);
define(AGGREGATOR_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_aggregator($get_options = TRUE) {
  if(module_exist('aggregator')) {
    $type = array(
      'name' => t('web headlines'),
      'description' => t('<b>Web Headlines</b>: All items for an aggregator category'),
      'include' => 'aggregator',    
      'prefix' => t(''),
      'suffix' => t('headlines'),
      'weight' => 10,
      'form' => FALSE,
      'label' => t('Add Web Headlines'),
      'help' => t('Some feeds are combined into categories. You can track an entire category in one box.'),
      'search' => TRUE,
      'threshold' => AGGREGATOR_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_aggregator_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type.
 */
function mysite_type_aggregator_options() {
  $options = array();
  $sql = "SELECT cid, title FROM {aggregator_category}";
  $result = db_query_range($sql, AGGREGATOR_FROM, AGGREGATOR_THRESHOLD);
  $aggregators = array();
  while($item = db_fetch_object($result)) {
    $aggregators[] =$item;
  }
  foreach ($aggregators as $key => $value) {
    $options['name'][] = $value->title;
    $options['type_id'][] = $value->cid;
    $options['type'][] = 'aggregator';
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_aggregator_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if (is_null($title)) {
      $sql = "SELECT title FROM {aggregator_category} WHERE cid = %d";
      $aggregator = db_fetch_object(db_query($sql, $type_id));
      $title = $aggregator->title;
    }  
    $type = mysite_type_aggregator();
    $title = $type['prefix'] . ' ' . $title . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_aggregator_data($type_id = NULL) {
  if(!empty($type_id)) {
    
    $sql = "SELECT cid, title FROM {aggregator_category} WHERE cid = %d";
    $feed = db_fetch_object(db_query($sql, $type_id));

    $data = array(
      'base' => 'aggregator/categories/' . $feed->cid,
      'xml' => 'aggregator/categories/' . $feed->cid . '/feed'
      );
      
    $sql = "SELECT i.iid, i.fid, i.title, i.link, i.author, i.description, i.timestamp FROM {aggregator_item} i INNER JOIN {aggregator_category_item} c ON c.iid = i.iid  WHERE c.cid = %d ORDER BY timestamp DESC";
    $result = db_query_range($sql, $feed->cid, 0, variable_get('mysite_elements', 5));
    $items = array();
    $i = 0;
    while ($obj = db_fetch_object($result)) {
      $sql = "SELECT fid, title FROM {aggregator_feed} WHERE fid = %d";
      $source = db_fetch_object(db_query($sql, $obj->fid));
      $items[$i]['type'] = 'aggregator';
      $items[$i]['link'] = l($obj->title, $obj->link);
      $items[$i]['title'] = check_plain($obj->title);
      $items[$i]['date'] = $obj->timestamp;
      $items[$i]['uid'] = NULL;
      $items[$i]['author'] = check_plain($source->title);
      $items[$i]['teaser'] = $obj->description;    
      $items[$i]['nid'] = NULL;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  
}
 
/**
 * Return block settings for this type
 */
function mysite_type_aggregator_block($arg, $op = 'view') {
  global $user;
  if (user_access('access news feeds') && $arg[0] == 'aggregator' && $arg[1] == 'categories' && is_numeric($arg[2])) {
    $sql = "SELECT cid, title FROM {aggregator_category} WHERE cid = %d";
    $aggregator = db_fetch_object(db_query($sql, $arg[2]));
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'aggregator';
    $data['type_id'] = $aggregator->cid;
    $data['title'] = mysite_type_aggregator_title($aggregator->cid, $aggregator->title);
    $content = mysite_block_handler($data);
    return $content;
  }
}
 
/**
 * Search options for this type
 */
function mysite_type_aggregator_search($uid = NULL) {
  if (!is_null($uid)) {
    $form['add_aggregator']['aggregator_title'] = array('#type' => 'textfield',
      '#title' => t('Headline Name'),
      '#default_value' => $edit['aggregator_title'],
      '#maxlength' => 64,
      '#description' => t('The name of the headlines you wish to add.'),
      '#required' => TRUE, 
      '#autocomplete_path' => 'autocomplete/mysite/aggregator'
    );  
    $form['add_aggregator']['uid'] = array('#type' => 'hidden', '#value' => $uid);
    $form['add_aggregator']['type'] = array('#type' => 'hidden', '#value' => 'aggregator');
    $form['add_aggregator']['submit'] = array('#type' => 'submit', '#value' =>t('Add aggregator'));  
    $output .= drupal_get_form('mysite_type_aggregator_search', $form);  
    return $output;
  }
}

/**
 * Form handler for search
 */
function mysite_type_aggregator_search_submit($form_id, $form_values) {
  // we use LIKE here in case JavaScript autocomplete support doesn't work.
  // or in case the user doesn't autocomplete the form
  $result = db_query("SELECT cid, title FROM {aggregator_category} WHERE title LIKE LOWER('%s%%')", $form_values['aggregator_title']);
  $count = 0;
  while ($aggregator = db_fetch_object($result)) {
    $data[$count]['uid'] = $form_values['uid'];
    $data[$count]['type'] = $form_values['type'];
    $data[$count]['type_id'] = $aggregator->cid;
    $data[$count]['title'] = mysite_type_aggregator_title($aggregator->cid, $aggregator->title);
    $data[$count]['description'] = t('The aggregator posts of %user', array('%user' => $aggregator->title));    
    $count++;
  }
  // pass the $data to the universal handler
  mysite_search_handler($data, 'aggregator');
  return;
}

/**
 * AJAX autocomplete for aggregator titles
 * The include files have no menu callbacks!
 */
function mysite_type_aggregator_autocomplete($string) {
  $matches = array();
  $result = db_query_range("SELECT cid, title FROM {aggregator_category} WHERE title LIKE LOWER('%s%%')", $string, 0, 10);
  while ($aggregator = db_fetch_object($result)) {
    $matches[$aggregator->title] = check_plain($aggregator->title);
  }
  return $matches;
}
 
/**
 * Cron cleanup function
 */
function mysite_type_aggregator_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT cid FROM {aggregator_category} WHERE cid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id']));
    if(empty($check->cid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}
