<?php
// $Id: blog.inc,v 1.1.2.5 2006/11/10 16:08:40 agentken Exp $

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

// static variables for this type
define(BLOG_THRESHOLD, 10);
define(BLOG_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_blog($get_options = TRUE) {
  if(module_exist('blog')) {
    $type = array(
      'name' => t('blogs'),
      'description' => t('<b>Blog Posts</b>: All blog posts from a specific user'),
      'include' => 'blog',    
      'prefix' => t(''),
      'suffix' => t('blog posts'),
      'weight' => -10,
      'form' => FALSE,
      'label' => t('Add User Blog'),
      'help' => t('You can choose blog postings by specific users. Type a user name in the search box, or choose from a recently active blogger.'),
      'search' => TRUE,
      'threshold' => BLOG_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_blog_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type
 */
function mysite_type_blog_options() {
  $options = array();
  // can't respect blog permissions because authenticated users aren't in the users_roles table
  // but it only pulls users who have blogged.
  $sql = "SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'blog' ORDER BY n.nid DESC";
  $result = db_query_range($sql, BLOG_FROM, BLOG_THRESHOLD);
  $blogs = array();
  while($item = db_fetch_object($result)) {
    $blogs[] =$item;
  }
  foreach ($blogs as $key => $value) {
    $options['name'][] = $value->name . t("'s");
    $options['type_id'][] = $value->uid;
    $options['type'][] = 'blog';
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_blog_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if (is_null($title)) {
      $blog = user_load(array('uid' => $type_id));
      $title = $blog->name;
    }  
    $type = mysite_type_blog();
    $title = $type['prefix'] . ' ' . $title . t("'s") . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_blog_data($type_id = NULL) {
  if(!empty($type_id)) {
    
    $sql = db_rewrite_sql("SELECT n.nid FROM {node} n WHERE n.type = 'blog' AND n.status = 1 AND n.uid = %d ORDER BY n.changed DESC");
    $result = db_query_range($sql, $type_id, 0, variable_get('mysite_elements', 5));

    $data = array(
      'base' => 'blog/' . $type_id,
      'xml' => 'blog/' . $type_id . '/feed',
      );
    $items = array();
    $i = 0;
    while ($nid = db_fetch_object($result)) {
      $node = node_load($nid->nid);
      $items[$i]['type'] = $node->type;
      $items[$i]['link'] = l($node->title, 'node/' . $nid->nid);
      $items[$i]['title'] = check_plain($node->title);
      $items[$i]['subtitle'] = NULL;      
      $items[$i]['date'] = $node->changed;
      $items[$i]['uid'] = $node->uid;
      $items[$i]['author'] = check_plain($node->name);
      $items[$i]['teaser'] = check_markup($node->teaser, $node->format);    
      $items[$i]['nid'] = $node->nid;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  
}

/**
 * Return block settings for this type
 */
function mysite_type_blog_block($arg, $op = 'view') {
  global $user;
  if (user_access('access content') && ($arg[0] == 'blog' || $arg[0] == 'user') && is_numeric($arg[1])) {
    $blog = user_load(array('uid' => $arg[1]));
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'blog';
    $data['type_id'] = $blog->uid;
    $data['title'] = mysite_type_blog_title($blog->uid, $blog->name);
    $content = mysite_block_handler($data);
    return $content;
  }
}

/**
 * Block rules for individual nodes
 */
function mysite_type_blog_block_node($nid = NULL) {
  if (!is_null($nid)) {
    global $user;
    $sql = "SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.nid = %d AND n.status = 1 AND n.type = 'blog'";
    $blog = db_fetch_object(db_query($sql, $nid));
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'blog';
    $data['type_id'] = $blog->uid;
    $data['title'] = mysite_type_blog_title($blog->uid, $blog->name);
    $content = mysite_block_handler($data);
    return $content;
  }  
}

/**
 * Search options for this type
 */
function mysite_type_blog_search($uid = NULL) {
  if (!is_null($uid)) {
    $form['add_blog']['blog_title'] = array('#type' => 'textfield',
      '#title' => t('User Name'),
      '#default_value' => $edit['blog_title'],
      '#maxlength' => 64,
      '#description' => t('The user name of the blog you wish to add.'),
      '#required' => FALSE, // this must be false, since all the search for are the same form
      '#autocomplete_path' => 'autocomplete/mysite/blog'
    );  
    $form['add_blog']['uid'] = array('#type' => 'hidden', '#value' => $uid);
    $form['add_blog']['type'] = array('#type' => 'hidden', '#value' => 'blog');
    $form['add_blog']['submit'] = array('#type' => 'submit', '#value' =>t('Add blog'));  
    $output .= drupal_get_form('mysite_type_blog_search', $form);  
    return $output;
  }
}

/**
 * Form handler for search
 */
function mysite_type_blog_search_submit($form_id, $form_values) {
  // we use LIKE here in case JavaScript autocomplete support doesn't work.
  // or in case the user doesn't autocomplete the form
  $result = db_query("SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'blog' AND u.name LIKE LOWER('%s%%')", $form_values['blog_title']);
  $count = 0;
  while ($blog = db_fetch_object($result)) {
    $data[$count]['uid'] = $form_values['uid'];
    $data[$count]['type'] = $form_values['type'];
    $data[$count]['type_id'] = $blog->uid;
    $data[$count]['title'] = mysite_type_blog_title($blog->uid, $blog->name);
    $data[$count]['description'] = t('The blog posts of %user', array('%user' => $blog->name));    
    $count++;
  }
  // pass the $data to the universal handler
  mysite_search_handler($data, 'blog');
  return;
}

/**
 * AJAX autocomplete for blog titles
 * The include files have no menu callbacks!
 */
function mysite_type_blog_autocomplete($string) {
  $matches = array();
  $result = db_query_range("SELECT DISTINCT(u.uid), u.name FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'blog' AND u.name LIKE LOWER('%s%%')", $string, 0, 10);
  while ($blog = db_fetch_object($result)) {
    $matches[$blog->name] = check_plain($blog->name);
  }
  return $matches;
}
 
/**
 * Cron cleanup function
 */
function mysite_type_blog_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT DISTINCT(u.uid) FROM {users} u INNER JOIN {node} n ON u.uid = n.uid WHERE n.status = 1 AND n.type = 'blog' AND u.uid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id']));
    if(empty($check->uid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}
 
