<?php
// $Id

// static variables for this type
define(BOOK_THRESHOLD, 10);
define(BOOK_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_book($get_options = TRUE) {
  if(module_exist('book')) {
    $type = array(
      'name' => t('books'),
      'description' => t('<b>Books</b>: Pages from a book collection'),
      'include' => 'book',    
      'prefix' => t(''),
      'suffix' => t('book pages'),
      'weight' => 0,
      'form' => FALSE,
      'label' => t('Add Book Updates'),
      'help' => t('You can track new pages for individual books. Type a book name in the search box, or choose from the list below.'),
      'search' => TRUE,
      'threshold' => BOOK_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_book_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type
 */
function mysite_type_book_options() {
  $options = array();
  $sql = "SELECT n.nid, n.title FROM {node} n INNER JOIN {book} b ON b.nid = n.nid WHERE b.parent =0 AND n.status = 1 AND n.type = 'book' ORDER BY n.title DESC";
  $result = db_query_range($sql, BOOK_FROM, BOOK_THRESHOLD);
  $books = array();
  while($item = db_fetch_object($result)) {
    $books[] = $item;
  }
  foreach ($books as $key => $value) {
    $options['name'][] = $value->title;
    $options['type_id'][] = $value->nid;
    $options['type'][] = 'book';
  }
  return $options;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_book_data($type_id = NULL) {
  if(!empty($type_id)) {
    $sql = db_rewrite_sql("SELECT n.nid FROM {node} n INNER JOIN {book} b ON n.nid = b.nid WHERE b.parent = %d AND n.type = 'book' AND n.status = 1 ORDER BY n.changed DESC, b.weight");
    $result = db_query_range($sql, $type_id, 0, variable_get('mysite_elements', 5));
    $data = array(
      'base' => 'book/' . $type_id,
      'xml' => 'book/' . $type_id . '/feed',
      );
    $items = array();
    $i = 0;
    while ($nid = db_fetch_object($result)) {
      $node = node_load($nid->nid);
      $items[$i]['type'] = $node->type;
      $items[$i]['link'] = l($node->title, 'node/' . $nid->nid);
      $items[$i]['title'] = check_plain($node->title);
      $items[$i]['subtitle'] = NULL;      
      $items[$i]['date'] = $node->changed;
      $items[$i]['uid'] = $node->uid;
      $items[$i]['author'] = check_plain($node->name);
      $items[$i]['teaser'] = check_markup($node->teaser, $node->format);    
      $items[$i]['nid'] = $node->nid;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  
}

/**
 * Return the title if given an id
 */
function mysite_type_book_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if (is_null($title)) {
      $book = node_load($type_id);
      $title = $book->title;
    }  
    $type = mysite_type_book();
    $title = $type['prefix'] . ' ' .$title . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 * Return node-specific block settings for this type
 */
function mysite_type_book_block_node($nid = NULL) {
  if (!is_null($nid)) {
    global $user;
    $result = db_query(db_rewrite_sql('SELECT n.nid, n.title, b.parent, n.type FROM {node} n INNER JOIN {book} b ON n.vid = b.vid WHERE n.nid = %d'), $nid);
    $book = db_fetch_object($result);
    // if no parent, this is the parent
    if ($book->parent == 0) {
      $book->parent = $book->nid;
    }
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'book';
    $data['type_id'] = $book->parent;
    $data['title'] = mysite_type_book_title($book->parent, $book->name);
    $content = mysite_block_handler($data);
    return $content;
  }
}

/**
 * Search options for this type
 */
function mysite_type_book_search($uid = NULL) {
  if (!is_null($uid)) {
    $form['add_book']['book_title'] = array('#type' => 'textfield',
      '#title' => t('Book Title'),
      '#default_value' => $edit['book_title'],
      '#maxlength' => 64,
      '#description' => t('The name of the book you wish to add.'),
      '#required' => TRUE, 
      '#autocomplete_path' => 'autocomplete/mysite/book'
    );  
    $form['add_book']['uid'] = array('#type' => 'hidden', '#value' => $uid);
    $form['add_book']['type'] = array('#type' => 'hidden', '#value' => 'book');
    $form['add_book']['submit'] = array('#type' => 'submit', '#value' =>t('Add book'));  
    $output .= drupal_get_form('mysite_type_book_search', $form);  
    return $output;
  }
}

/**
 * Form handler for search
 */
function mysite_type_book_search_submit($form_id, $form_values) {
  // we use LIKE here in case JavaScript autocomplete support doesn't work.
  // or in case the user doesn't autocomplete the form
  $sql = db_rewrite_sql("SELECT n.nid, n.title FROM {node} n INNER JOIN {book} b ON b.nid = n.nid WHERE LOWER(n.title) LIKE LOWER('%s%%') AND b.parent = 0");
  $result = db_query($sql, $form_values['book_title'], $book);
  $count = 0;
  while ($book = db_fetch_object($result)) {
    $data[$count]['uid'] = $form_values['uid'];
    $data[$count]['type'] = $form_values['type'];
    $data[$count]['type_id'] = $book->nid;
    $data[$count]['title'] = mysite_type_book_title($book->nid, $book->title);
    $data[$count]['description'] = $book->title;    
    $count++;
  }
  // pass the $data to the universal handler
  mysite_search_handler($data, 'book');
  return;
}

/**
 * AJAX autocomplete for book titles
 * The include files have no menu callbacks!
 */
function mysite_type_book_autocomplete($string) {
  $matches = array();
  $sql = db_rewrite_sql("SELECT n.nid, n.title FROM {node} n INNER JOIN {book} b ON b.nid = n.nid WHERE LOWER(n.title) LIKE LOWER('%s%%') AND b.parent = 0");  
  $result = db_query_range($sql, $string, 0, 10);
  while ($book = db_fetch_object($result)) {
    $matches[$book->title] = check_plain($book->title);
  }
  return $matches;
}

/**
 * Cron cleanup function
 */
function mysite_type_book_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT b.nid FROM {book} b WHERE b.parent = 0 AND b.nid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id']));
    if(empty($check->nid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}
