<?php
// $Id: feed.inc,v 1.1.2.3 2006/12/14 15:48:12 agentken Exp $

/**
 * These files are included as needed to reduce overhead.
 * See the README for additional information.
 */

// static variables for this type
define(FEED_THRESHOLD, 10);
define(FEED_FROM, 0);

/**
 * Basic definition of the content type. API element.
 */
function mysite_type_feed($get_options = TRUE) {
  if(module_exist('aggregator')) {
    $type = array(
      'name' => t('web feeds'),
      'description' => t('<b>Web Feeds</b>: All items for a specific aggregator feed'),
      'include' => 'feed',    
      'prefix' => t(''),
      'suffix' => t('feed'),
      'weight' => 8,
      'form' => TRUE,
      'label' => t('Add Existing Web Feed'),
      'help' => t('Some useful web feeds are available for you to choose. Use the search box, or select an example.'),
      'search' => TRUE,
      'threshold' => FEED_THRESHOLD
    );
    if ($get_options) {
      $type['options'] = mysite_type_feed_options();
    }
    return $type;
  }
}

/**
 * Set the options for this type
 */
function mysite_type_feed_options() {
  $options = array();
  $feeds = array();

  // get the categories
  $sql = "SELECT cid, title FROM {aggregator_category}";
  $result = db_query($sql);
  while ($cat = db_fetch_object($result)) {
    $sql = "SELECT a.fid, a.title FROM {aggregator_feed} a INNER JOIN {aggregator_category_feed} c ON a.fid = c.fid WHERE c.cid = %d";
    $new_result = db_query_range($sql, $cat->cid, FEED_FROM, FEED_THRESHOLD);
    while($item = db_fetch_object($new_result)) {
      $feeds[$cat->title][] = $item;
    }
  }
  foreach ($feeds as $key => $value) {
    foreach ($value as $item) {
      $options['group'][] = $key;
      $options['name'][] = $item->title;
      $options['type_id'][] = $item->fid;
      $options['type'][] = 'feed';
    }  
  }
  return $options;
}

/**
 * Return the title if given an id
 */
function mysite_type_feed_title($type_id = NULL, $title = NULL) {
  if(!empty($type_id)) {
    if (is_null($title)) {
      $sql = "SELECT title FROM {aggregator_feed} WHERE fid = %d";
      $feed = db_fetch_object(db_query($sql, $type_id));
      $title = $feed->title;
    }  
    $type = mysite_type_feed();
    $title = $type['prefix'] . ' ' . $title . ' ' . $type['suffix'];
    $title = trim(rtrim($title));
    return $title;
  }
  drupal_set_message(t('Could not find title'), 'error');
  return;
}

/**
 *  Return the page data for a requested id
 */
function mysite_type_feed_data($type_id = NULL) {
  if(!empty($type_id)) {
    
    $sql = "SELECT fid, title, url, image, checked FROM {aggregator_feed} WHERE fid = %d";
    $feed = db_fetch_object(db_query($sql, $type_id));

    $data = array(
      'base' => 'aggregator/sources/' . $feed->fid,
      'xml' => $feed->url,
      'image' => $feed->image
      );
      
    $sql = "SELECT iid, title, link, author, description, timestamp FROM {aggregator_item} WHERE fid = %d ORDER BY timestamp DESC";
    $result = db_query_range($sql, $feed->fid, 0, variable_get('mysite_elements', 5));
    $items = array();
    $i = 0;
    while ($obj = db_fetch_object($result)) {
      $items[$i]['type'] = 'feed';
      $items[$i]['link'] = l($obj->title, $obj->link);
      $items[$i]['title'] = check_plain($obj->title);
      $items[$i]['subtitle'] = NULL;      
      $items[$i]['date'] = $obj->timestamp;
      $items[$i]['uid'] = NULL;
      $items[$i]['author'] = check_plain($obj->author);
      $items[$i]['teaser'] = $obj->description;     
      $items[$i]['nid'] = NULL;
      $i++;
    }
    $data['items'] = $items;
    return $data;
  }
  drupal_set_message(t('Could not find data'), 'error');
  return;  

}
 
/**
 * Return block settings for this type
 */
function mysite_type_feed_block($arg, $op = 'view') {
  global $user;
  if (user_access('access news feeds') && $arg[0] == 'aggregator' && $arg[1] == 'sources' && is_numeric($arg[2])) {
    $sql = "SELECT fid, title FROM {aggregator_feed} WHERE fid = %d";
    $feed = db_fetch_object(db_query($sql, $arg[2]));
    $data = array();
    $data['uid'] = $user->uid;
    $data['type'] = 'feed';
    $data['type_id'] = $feed->fid;
    $data['title'] = $feed->title;
    $content = mysite_block_handler($data);
    return $content;
  }   
}  

/**
 * Search options for this type
 */
function mysite_type_feed_search($uid = NULL) {
  if (!is_null($uid)) {
    $output = '';
    // we cannot use 'title' here because of autocomplete form handler conflicts
    $form['add_feed']['feed_title'] = array('#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $edit['feed_title'],
      '#maxlength' => 64,
      '#description' => t('The name of the feed; typically the name of the web site you syndicate content from.'),
      '#required' => FALSE, // this must be false, since all the search for are the same form
      '#autocomplete_path' => 'autocomplete/mysite/feed'
    );  
    $form['add_feed']['uid'] = array('#type' => 'hidden', '#value' => $uid);
    $form['add_feed']['type'] = array('#type' => 'hidden', '#value' => 'feed');
    $form['add_feed']['submit'] = array('#type' => 'submit', '#value' =>t('Add Feed'));  
    $output .= drupal_get_form('mysite_type_feed_search', $form);  
    return $output;
  }
}

/**
 * Form handler for search
 */
function mysite_type_feed_search_submit($form_id, $form_values) {
  // we use LIKE here in case JavaScript autocomplete support doesn't work.
  // or in case the user doesn't autocomplete the form
  $result = db_query("SELECT fid, title, description FROM {aggregator_feed} WHERE title LIKE LOWER('%s%%')", $form_values['feed_title']);
  $count = 0;
  while ($feed = db_fetch_object($result)) {
    $data[$count]['uid'] = $form_values['uid'];
    $data[$count]['type'] = $form_values['type'];
    $data[$count]['type_id'] = $feed->fid;
    $data[$count]['title'] = mysite_type_feed_title($feed->fid, $feed->title);
    $data[$count]['description'] = $feed->description;    
    $count++;
  }
  // pass the $data to the universal handler
  mysite_search_handler($data, 'feed');
  return;
}

/**
 * AJAX autocomplete for aggregator titles
 * The include files have no menu callbacks!
 */
function mysite_type_feed_autocomplete($string) {
  $matches = array();
  $result = db_query_range("SELECT title FROM {aggregator_feed} WHERE title LIKE LOWER('%s%%')", $string, 0, 10);
  while ($feed = db_fetch_object($result)) {
    $matches[$feed->title] = check_plain($feed->title);
  }
  return $matches;
}

/**
 * Cron cleanup function
 */
function mysite_type_feed_clear($type) {
  // fetch all the active records of this type and see if they really exist in the proper table
  $sql = "SELECT mid, uid, type_id, title FROM {mysite_data} WHERE type = '%s'";
  $result = db_query($sql, $type);
  $data = array();
  while ($item = db_fetch_array($result)) {
    $sql = "SELECT fid FROM {aggregator_feed} WHERE fid = %d";
    $check = db_fetch_object(db_query($sql, $item['type_id']));
    if(empty($check->fid)) {
      $data[$item['mid']]  = $item;
    }  
  }
  return $data;
}

/**
 * Settings for this type
 */
function mysite_type_feed_settings() {
  // specific rules for the aggregator module.  Since we let users add feeds, we need to preset some default values for each feed.
  if (module_exist('aggregator')) {
    $form['mysite_feed'] = array(
      '#type' => 'fieldset',
      '#title' => t('Aggregator Settings'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ); 
    $use_feeds = variable_get('mysite_feed', 1);
    $form['mysite_feed']['mysite_feed'] = array(
      '#type' => 'radios', '#title' => t('Aggregator Feeds'), '#default_value' => $use_feeds,
      '#options' =>  array(1 => t('Allow users to add new feeds'), 0 => t('Do not allow users to add new feeds')),
      '#description' => t('Should users be able to add feeds to their mysite?  If so, be sure to grant the "add mysite feeds" permission in <a href="%url">mysite access control</a>.', array('%url' => url('admin/access')))
    );
    if ($use_feeds == 1 && user_access('administer news feeds')) {
      $sql = "SELECT cid, title, description FROM {aggregator_category}";
      $result = db_query($sql);
      $options = array();
      while ($feed = db_fetch_object($result)) {
        $options[$feed->cid] = $feed->title;
      }  
      if (empty($options)) {
       $form['mysite_feed']['no_feeds'] = array(
         '#value' => t('<p>There are no categories configured.  Please go to <a href="%url">aggregator categories</a> to add one. <br /><b>User Feeds</b> is a good choice.</p>', array('%url' => url('admin/aggregator/add/category'))) ,
        );      
      }
      else {
        $form['mysite_feed']['mysite_feed_categories'] = array(
          '#type' => 'checkboxes', '#title' => t('Allowed Feed Categories'), '#default_value' => variable_get('mysite_feed_categories', NULL),
          '#options' =>  $options,
          '#description' => t('In which categories may users add new feeds?')
        );
      $feeds = variable_get('mysite_feed_categories', NULL);
      if (!empty($feeds)) {
        $implode = "(" . implode(", ", $feeds) . ")";
        $sql = "SELECT cid, title, description FROM {aggregator_category} WHERE cid IN %s";
        $result = db_query($sql, $implode);  
      }
      else {
       $form['mysite_feed']['no_categories'] = array(
         '#value' => t('<p>There are no categories configured. After you select at least one category, you may select a default.', array('%url' => url('admin/aggregator/add/category'))) ,
        );      
      }
      $options = array();
      while ($feed = db_fetch_object($result)) {
        $options[$feed->cid] = $feed->title;
      }          
        $form['mysite_feed']['mysite_feed_default'] = array(
          '#type' => 'radios', '#title' => t('Default Feed Category'), '#default_value' => variable_get('mysite_feed_default', NULL),
          '#options' =>  $options,
          '#required' => FALSE,
          '#description' => t('What is the default category for new feeds?')
        );        
      }  
      $period = drupal_map_assoc(array(900, 1800, 3600, 7200, 10800, 21600, 32400, 43200, 64800, 86400, 172800, 259200, 604800, 1209600, 2419200), 'format_interval');
      $form['mysite_feed']['mysite_feed_update'] = array(
        '#type' => 'select', '#title' => t('Update interval'), '#default_value' => variable_get('mysite_feed_update', 3600),
        '#options' => $period,
        '#description' => t('Define the update interval for user-supplied feeds.  This value will be used for all user-submitted feeds.')
      );
    }
    return system_settings_form('mysite_feed', $form);  
  }
  else {
    return t('The aggregator module is not active.  These features cannot be used.');
  }
}


/**
 * Form element distinct to this type
 * Add an XML feed direcly through mysite
 */
function mysite_type_feed_form($owner) {
  if(empty($owner)) {
    global $user;
    $owner = $user;
  }
  $output = '<p>' . t('You aren\'t limited to existing feeds. Add any RSS or Atom webfeed from your favorite sites.') . '</p>';
  if (user_access('add mysite feeds')) {
    $feeds = variable_get('mysite_feed_categories', NULL);
    $refresh = variable_get('mysite_feed_update', 3600);
     $form['new_feed'] = array(
      '#type' => 'fieldset',
      '#title' => t('Add New Web Feed'),
      '#weight' => 5,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['new_feed']['title'] = array('#type' => 'textfield',
      '#title' => t('Title'),
      '#default_value' => $edit['title'],
      '#maxlength' => 64,
      '#description' => t('The name of the feed; typically the name of the web site you syndicate content from.'),
      '#required' => FALSE,
      '#autocomplete_path' => 'autocomplete/mysite/feed'
    );
    $form['new_feed']['url'] = array('#type' => 'textfield',
      '#title' => t('URL'),
      '#default_value' => $edit['url'],
      '#maxlength' => 255,
      '#description' => t('The fully-qualified URL of the feed.'),
      '#required' => FALSE,
    );
    $form['new_feed']['refresh'] = array(
        '#type' => 'hidden', '#value' => $refresh
    );  
    $form['new_feed']['uid'] = array(
        '#type' => 'hidden', '#value' => $owner->uid
    );  
    // categories
    if ($_POST) {
      $edit = $_POST['edit'];
    }
    else {
      $edit['category'] = variable_get('mysite_feed_default', NULL);
    }
    $implode = "(" . implode(", ", $feeds) . ")";
    $sql = "SELECT cid, title, description FROM {aggregator_category} WHERE cid IN %s";
    $result = db_query($sql, $implode);  
    $options = array();
    while ($category = db_fetch_object($result)) {
      $options[$category->cid] = check_plain($category->title);
    }  
    if (empty($options)) {
     $form['new_feed']['no_feeds'] = array(
       '#value' => t('<p>There are no categories configured.  Please go to <a href="%url">aggregator categories</a> to add one. <br /><b>User Feeds</b> is a good choice.</p>', array('%url' => url('admin/aggregator/add/category'))) ,
      );      
    }
    else if (count($options) > 1) {
      $form['new_feed']['category'] = array(
        '#type' => 'radios', '#title' => t('Feed Categories'), '#default_value' => $edit['category'],
        '#options' =>  $options,
        '#description' => t('Select a category for this feed')
      );
    }    
    else {
      $form['new_feed']['category'] = array(
        '#type' => 'hidden', '#title' => t('Feed Categories'), '#default_value' => $edit['category']
      );  
    }
    $form['new_feed']['submit'] = array('#type' => 'submit', '#value' =>t('Submit New Feed'));  
  
    $output .= drupal_get_form('mysite_type_feed_form', $form);
  }  
  return $output;
}

/**
 * Handler for mysite_type_feed_form
 */
function mysite_type_feed_form_submit($form_id, $form_values) {
  if ($_POST['op'] == t('Submit New Feed')) {
    // Check for duplicate titles and see what we need to save
    if (isset($form_values['fid'])) {
      $result = db_query("SELECT fid, title, url, description FROM {aggregator_feed} WHERE (title = '%s' OR url='%s') AND fid != %d", $form_values['title'], $form_values['url'], $form_values['fid']);
    }
    else {
      $result = db_query("SELECT fid, title, url, description FROM {aggregator_feed} WHERE title = '%s' OR url='%s'", $form_values['title'], $form_values['url']);
    }
    $success = FALSE;
    while ($feed = db_fetch_object($result)) {
      $form_values['url'] = $feed->url;    
      if ($form_values['title'] && $form_values['url'] && strcasecmp($feed->title, $form_values['title']) == 0) {
        $check = mysite_check($form_values['uid'], 'feed', $feed->fid, $mid = NULL);
        unset ($_SESSION['messages']['error']);
        if ($check == 0) {
          drupal_set_message(t('A feed named %feed already exists. <a href="%url">Would you like to add it to your mysite</a>?', array('%feed' => theme('placeholder', $form_values['title']), '%url' => url(mysite_add_link($form_values['uid'], 'feed', $feed->fid)))), 'status');
        }
        else {
          drupal_set_message(t('A feed named %feed already exists and has been added to your mysite.', array('%feed' => theme('placeholder', $form_values['title']), '%url' => url(mysite_add_link($form_values['uid'], 'feed', $feed->fid)))), 'status');
        }
        $success = TRUE;
        $form_values['url'] = NULL;
        $form_values['title'] = NULL;
      }
    }
    if (!$success) {
      if (empty($form_values['title'])) {
        form_set_error('title', t('You must enter a valid title.'));      
      }
      if (empty($form_values['url'])) {
        form_set_error('url', t('You must enter a valid URL.'));      
      }
    }
  }

  // if we need to save data, then we will
  if (!empty($form_values['title']) && !empty($form_values['uid'])) {
    // check the validity of the feed source
    // Request feed.
    $headers = array();
    $modified = 0;
    $headers['If-Modified-Since'] = gmdate('D, d M Y H:i:s', $modified) .' GMT';    
    $result = drupal_http_request($form_values['url'], $headers);
    // we must make sure that the feed is XML -- why doesn't aggregator do this?
    if (!stristr($result->headers['Content-Type'], 'xml') && !stristr($result->headers['Content-type'], 'xml')) {
      drupal_set_message(t('<p>Failed to find RSS feed.  Please check the URL and try again.</p>'), 'error');
      return;
    }
    else {
      // Process HTTP response code.
      switch ($result->code) {    
        case 301:
          $form_values['url'] = $result->redirect_url;
          $continue = TRUE;        
          break;
        case 304:
        case 200:
        case 302:
        case 307:
          $continue = TRUE;
          break;
        default:    
          $continue = FALSE;
          drupal_set_message (t('<p>Failed to parse RSS feed.  Please check the URL and try again.</p>'), 'error');
          return;
      }  
    }
    // check the status of a corrected feed
    $result = db_query("SELECT fid, title, url, description FROM {aggregator_feed} WHERE title = '%s' OR url='%s'", $form_values['title'], $form_values['url']);
    $feed_check = db_fetch_object($result);
    if (empty($feed_check)) {
      $continue = TRUE;
    }
    else {
      $continue = FALSE;
      $check = mysite_check($form_values['uid'], 'feed', $feed_check->fid, $mid = NULL);
      unset ($_SESSION['messages']['error']);
      if ($check == 0) {
        drupal_set_message(t('A feed named %feed already exists at that url. <a href="%url">Would you like to add it to your mysite</a>?', array('%feed' => theme('placeholder', $feed_check->title), '%url' => url(mysite_add_link($form_values['uid'], 'feed', $feed_check->fid)))), 'status');
      }
      else {
        drupal_set_message(t('A feed named %feed already exists at that url and has been added to your mysite.', array('%feed' => theme('placeholder', $feed_check->title), '%url' => url(mysite_add_link($form_values['uid'], 'feed', $feed_check->fid)))), 'status');
      }      
    }
    if ($continue) {
      // A single unique id for bundles and feeds, to use in blocks.
      $form_values['fid'] = db_next_id('{aggregator_feed}_fid');
      db_query("INSERT INTO {aggregator_feed} (fid, title, url, refresh, etag, modified, block) VALUES (%d, '%s', '%s', %d, '%s', %d, 5)", $form_values['fid'], $form_values['title'], $form_values['url'], $form_values['refresh'], $result->headers['ETag'], $modified);
      // The feed is being saved, save the categories as well.
      if ($form_values['category']) {
        db_query('INSERT INTO {aggregator_category_feed} (fid, cid) VALUES (%d, %d)', $form_values['fid'], $form_values['category']);
      }
      menu_rebuild();
      watchdog('aggregator', t('Feed %feed added.', array('%feed' => theme('placeholder', $form_values['title']))), WATCHDOG_NOTICE, l(t('view'), 'admin/aggregator'));
      drupal_set_message(t('The feed %feed has been added.', array('%feed' => theme('placeholder', $form_values['title']))));

      // refresh the feed to add content
      aggregator_refresh(aggregator_get_feed($form_values['fid']));
      
      // save to mysite
      mysite_content_add($form_values['uid'], 'feed', $form_values['fid']);
    }
  }
}
